package dev.inmo.postssystem.features.auth.client

import dev.inmo.postssystem.features.auth.common.*
import dev.inmo.postssystem.features.users.common.User
import dev.inmo.micro_utils.ktor.client.bodyOrNull
import dev.inmo.micro_utils.ktor.common.buildStandardUrl
import io.ktor.client.HttpClient
import io.ktor.client.request.*
import kotlinx.serialization.builtins.nullable

class ClientAuthFeature(
    private val client: HttpClient,
    baseUrl: String
) : AuthFeature {
    private val rootUrl = buildStandardUrl(baseUrl.dropLastWhile { it == '/' }, authRootPathPart)
    private val fullAuthPath = buildStandardUrl(
        rootUrl,
        authAuthPathPart
    )
    private val fullRefreshPath = buildStandardUrl(
        rootUrl,
        authRefreshPathPart
    )
    private val fullGetMePath = buildStandardUrl(
        rootUrl,
        authGetMePathPart
    )

    override suspend fun auth(creds: AuthCreds): AuthTokenInfo? = client.post(
        fullAuthPath
    ) {
        setBody(creds)
    }.bodyOrNull()

    override suspend fun refresh(refresh: RefreshToken): AuthTokenInfo? = client.post(
        fullRefreshPath
    ) {
        setBody(refresh)
    }.bodyOrNull()

    override suspend fun getMe(authToken: AuthToken): User? = client.post(
        fullGetMePath
    ) {
        setBody(authToken)
    }.bodyOrNull()

    fun isAuthRequest(builder: HttpRequestBuilder): Boolean = builder.url.buildString().let {
        it == fullAuthPath || it == fullRefreshPath
    }
}
