package dev.inmo.postssystem.features.auth.client.ui

import dev.inmo.postssystem.features.auth.client.settings.AuthSettings
import dev.inmo.postssystem.features.auth.common.AuthCreds
import dev.inmo.postssystem.features.common.common.ui.AbstractUIModel
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.launch

class DefaultAuthUIModel(
    private val scope: CoroutineScope,
    private val authSettings: AuthSettings
) : AbstractUIModel<AuthUIState>(AuthUIState.Loading), AuthUIModel {
    init {
        scope.launch {
            _currentState.value = AuthUIState.Loading
            authSettings.loadingJob.join()
            if (authSettings.authorizedDIModule.value == null) {
                _currentState.value = AuthUIState.DefaultInit
            } else {
                _currentState.value = AuthUIState.Authorized
            }
        }
    }

    override suspend fun initAuth(serverUrl: String, creds: AuthCreds) {
        _currentState.value = AuthUIState.Loading
        val authError = authSettings.auth(serverUrl, creds)
        if (authError == null) {
            _currentState.value = AuthUIState.Authorized
        } else {
            _currentState.value = AuthUIState.Init(authError)
        }
    }

}
