package dev.inmo.postssystem.features.content.binary.server

import com.benasher44.uuid.uuid4
import com.sksamuel.scrimage.ImmutableImage
import com.sksamuel.scrimage.nio.JpegWriter
import dev.inmo.postssystem.features.common.common.SimpleInputProvider
import io.ktor.util.asStream
import io.ktor.utils.io.core.Input
import io.ktor.utils.io.streams.asInput
import kotlinx.coroutines.currentCoroutineContext
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.isActive
import java.io.File

class ScrimageBasedImagesCropper(
    private val config: PreviewImageDimensionsConfig
) : ImagesCropper {
    override suspend fun crop(inputProvider: SimpleInputProvider): Flow<Input> = flow {
        val outputTempFile = File.createTempFile(uuid4().toString(), ".temp").apply {
            createNewFile()
            deleteOnExit()
        }
        ImmutableImage.loader().fromStream(
            inputProvider().asStream()
        ).max(
            config.maxWidth,
            config.maxHeight
        ).let {
            if (currentCoroutineContext().isActive) {
                it.output(
                    JpegWriter(),
                    outputTempFile
                )
            } else {
                return@flow
            }
        }

        val input = outputTempFile.inputStream().asInput()
        runCatching {
            emit(input)
        }.onSuccess {
            if (input.endOfInput) { // remove file if it was fully read inside of emit
                outputTempFile.delete()
            }
        }.onFailure {
            outputTempFile.delete()
        }
    }
}
