package dev.inmo.postssystem.features.files.client

import dev.inmo.postssystem.features.files.common.*
import dev.inmo.postssystem.features.files.common.storage.ReadFilesStorage
import dev.inmo.micro_utils.ktor.client.bodyOrNull
import dev.inmo.micro_utils.ktor.common.buildStandardUrl
import dev.inmo.micro_utils.repos.ReadCRUDRepo
import dev.inmo.micro_utils.repos.ktor.client.crud.KtorReadCRUDRepoClient
import io.ktor.client.HttpClient
import io.ktor.client.request.*
import io.ktor.client.statement.readBytes
import io.ktor.http.ContentType
import io.ktor.http.encodeURLQueryComponent
import kotlinx.serialization.BinaryFormat
import kotlinx.serialization.builtins.nullable

class ClientReadFilesStorage(
    baseUrl: String,
    private val client: HttpClient,
    private val serialFormat: BinaryFormat,
    contentType: ContentType
) : ReadFilesStorage, ReadCRUDRepo<MetaFileInfoStorageWrapper, FileId> by KtorReadCRUDRepoClient<MetaFileInfoStorageWrapper, FileId>(
    buildStandardUrl(baseUrl, filesRootPathPart),
    client,
    contentType,
    { it.string.encodeURLQueryComponent() }
) {
    private val fullFilesPath = buildStandardUrl(baseUrl, filesRootPathPart)
    private val fullFilesGetBytesPath = buildStandardUrl(
        fullFilesPath,
        filesGetFilesPathPart
    )

    override suspend fun getBytes(id: FileId): ByteArray = client.post(fullFilesGetBytesPath) {
        setBody(serialFormat.encodeToByteArray(FileId.serializer(), id))
    }.readBytes()

    override suspend fun getFullFileInfo(
        id: FileId
    ): FullFileInfoStorageWrapper? = client.get(
        buildStandardUrl(
            fullFilesPath,
            filesGetFullFileInfoPathPart,
            filesFileIdParameter to id.string.encodeURLQueryComponent()
        )
    ).bodyOrNull()
}
